from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ManagementInstanceSessionsEventsGetOutputConnectionUsage:
    total_productive_client_message_count: float
    total_productive_provider_message_count: float
@dataclass
class ManagementInstanceSessionsEventsGetOutputConnectionMcp:
    capabilities: Dict[str, Any]
    protocol_version: str
    transport: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputConnectionParticipantData:
    identifier: str
    name: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputConnectionParticipant:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    data: ManagementInstanceSessionsEventsGetOutputConnectionParticipantData
    created_at: datetime
    provider_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputConnection:
    object: str
    id: str
    connection_state: str
    transport: str
    usage: ManagementInstanceSessionsEventsGetOutputConnectionUsage
    session_id: str
    has_errors: bool
    has_warnings: bool
    created_at: datetime
    last_message_at: datetime
    mcp: Optional[ManagementInstanceSessionsEventsGetOutputConnectionMcp] = None
    participant: Optional[ManagementInstanceSessionsEventsGetOutputConnectionParticipant] = None
    last_active_at: Optional[datetime] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputProviderRun:
    object: str
    id: str
    status: str
    session_id: str
    session_provider_id: str
    provider_id: str
    connection_id: str
    created_at: datetime
    updated_at: datetime
    completed_at: Optional[datetime] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageHierarchy:
    object: str
    type: str
    child_message_ids: List[str]
    parent_message_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageTransportMcp:
    object: str
    id: Union[str, float]
    protocol_version: str
    transport: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageTransportToolCall:
    object: str
    id: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageTransport:
    object: str
    type: str
    mcp: Optional[ManagementInstanceSessionsEventsGetOutputMessageTransportMcp] = None
    tool_call: Optional[ManagementInstanceSessionsEventsGetOutputMessageTransportToolCall] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags:
    destructive: Optional[bool] = None
    read_only: Optional[bool] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCallTool:
    object: str
    id: str
    key: str
    name: str
    capabilities: Dict[str, Any]
    constraints: List[str]
    instructions: List[str]
    specification_id: str
    provider_id: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    input_schema: Optional[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema] = None
    output_schema: Optional[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema] = None
    tags: Optional[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCallError:
    object: str
    id: str
    code: str
    message: str
    data: Dict[str, Any]
    status: str
    session_id: str
    similar_error_count: float
    created_at: datetime
    provider_run_id: Optional[str] = None
    connection_id: Optional[str] = None
    group_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageToolCall:
    object: str
    id: str
    tool_key: str
    type: str
    status: str
    source: str
    transport: str
    session_id: str
    message_id: str
    tool: ManagementInstanceSessionsEventsGetOutputMessageToolCallTool
    created_at: datetime
    session_provider_id: Optional[str] = None
    connection_id: Optional[str] = None
    provider_run_id: Optional[str] = None
    error: Optional[ManagementInstanceSessionsEventsGetOutputMessageToolCallError] = None
    input: Optional[Dict[str, Any]] = None
    output: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData:
    identifier: str
    name: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageSenderParticipant:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    data: ManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData
    created_at: datetime
    provider_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData:
    identifier: str
    name: str
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageResponderParticipant:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    data: ManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData
    created_at: datetime
    provider_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessageError:
    object: str
    id: str
    code: str
    message: str
    data: Dict[str, Any]
    status: str
    session_id: str
    similar_error_count: float
    created_at: datetime
    provider_run_id: Optional[str] = None
    connection_id: Optional[str] = None
    group_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputMessage:
    object: str
    id: str
    type: str
    status: str
    source: str
    session_id: str
    hierarchy: ManagementInstanceSessionsEventsGetOutputMessageHierarchy
    transport: ManagementInstanceSessionsEventsGetOutputMessageTransport
    sender_participant: ManagementInstanceSessionsEventsGetOutputMessageSenderParticipant
    created_at: datetime
    session_provider_id: Optional[str] = None
    connection_id: Optional[str] = None
    provider_run_id: Optional[str] = None
    input: Optional[Dict[str, Any]] = None
    output: Optional[Dict[str, Any]] = None
    tool_call: Optional[ManagementInstanceSessionsEventsGetOutputMessageToolCall] = None
    responder_participant: Optional[ManagementInstanceSessionsEventsGetOutputMessageResponderParticipant] = None
    error: Optional[ManagementInstanceSessionsEventsGetOutputMessageError] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputError:
    object: str
    id: str
    code: str
    message: str
    data: Dict[str, Any]
    status: str
    session_id: str
    similar_error_count: float
    created_at: datetime
    provider_run_id: Optional[str] = None
    connection_id: Optional[str] = None
    group_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutputWarning:
    object: str
    id: str
    code: str
    message: str
    data: Dict[str, Any]
    session_id: str
    created_at: datetime
    connection_id: Optional[str] = None
@dataclass
class ManagementInstanceSessionsEventsGetOutput:
    object: str
    id: str
    type: str
    session_id: str
    created_at: datetime
    connection: Optional[ManagementInstanceSessionsEventsGetOutputConnection] = None
    provider_run: Optional[ManagementInstanceSessionsEventsGetOutputProviderRun] = None
    message: Optional[ManagementInstanceSessionsEventsGetOutputMessage] = None
    error: Optional[ManagementInstanceSessionsEventsGetOutputError] = None
    warning: Optional[ManagementInstanceSessionsEventsGetOutputWarning] = None


class mapManagementInstanceSessionsEventsGetOutputConnectionUsage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputConnectionUsage:
        return ManagementInstanceSessionsEventsGetOutputConnectionUsage(
        total_productive_client_message_count=data.get('total_productive_client_message_count'),
        total_productive_provider_message_count=data.get('total_productive_provider_message_count')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputConnectionUsage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputConnectionMcp:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputConnectionMcp:
        return ManagementInstanceSessionsEventsGetOutputConnectionMcp(
        capabilities=data.get('capabilities'),
        protocol_version=data.get('protocol_version'),
        transport=data.get('transport')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputConnectionMcp, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputConnectionParticipantData:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputConnectionParticipantData:
        return ManagementInstanceSessionsEventsGetOutputConnectionParticipantData(
        identifier=data.get('identifier'),
        name=data.get('name')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputConnectionParticipantData, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputConnectionParticipant:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputConnectionParticipant:
        return ManagementInstanceSessionsEventsGetOutputConnectionParticipant(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        data=mapManagementInstanceSessionsEventsGetOutputConnectionParticipantData.from_dict(data.get('data')) if data.get('data') else None,
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputConnectionParticipant, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputConnection:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputConnection:
        return ManagementInstanceSessionsEventsGetOutputConnection(
        object=data.get('object'),
        id=data.get('id'),
        connection_state=data.get('connection_state'),
        transport=data.get('transport'),
        usage=mapManagementInstanceSessionsEventsGetOutputConnectionUsage.from_dict(data.get('usage')) if data.get('usage') else None,
        mcp=mapManagementInstanceSessionsEventsGetOutputConnectionMcp.from_dict(data.get('mcp')) if data.get('mcp') else None,
        session_id=data.get('session_id'),
        participant=mapManagementInstanceSessionsEventsGetOutputConnectionParticipant.from_dict(data.get('participant')) if data.get('participant') else None,
        has_errors=data.get('has_errors'),
        has_warnings=data.get('has_warnings'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        last_message_at=datetime.fromisoformat(data.get('last_message_at').replace('Z', '+00:00')) if data.get('last_message_at') else None,
        last_active_at=datetime.fromisoformat(data.get('last_active_at').replace('Z', '+00:00')) if data.get('last_active_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputConnection, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputProviderRun:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputProviderRun:
        return ManagementInstanceSessionsEventsGetOutputProviderRun(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        session_id=data.get('session_id'),
        session_provider_id=data.get('session_provider_id'),
        provider_id=data.get('provider_id'),
        connection_id=data.get('connection_id'),
        completed_at=datetime.fromisoformat(data.get('completed_at').replace('Z', '+00:00')) if data.get('completed_at') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputProviderRun, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageHierarchy:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageHierarchy:
        return ManagementInstanceSessionsEventsGetOutputMessageHierarchy(
        object=data.get('object'),
        type=data.get('type'),
        parent_message_id=data.get('parent_message_id'),
        child_message_ids=data.get('child_message_ids', [])
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageHierarchy, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageTransportMcp:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageTransportMcp:
        return ManagementInstanceSessionsEventsGetOutputMessageTransportMcp(
        object=data.get('object'),
        id=data.get('id'),
        protocol_version=data.get('protocol_version'),
        transport=data.get('transport')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageTransportMcp, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageTransportToolCall:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageTransportToolCall:
        return ManagementInstanceSessionsEventsGetOutputMessageTransportToolCall(
        object=data.get('object'),
        id=data.get('id')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageTransportToolCall, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageTransport:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageTransport:
        return ManagementInstanceSessionsEventsGetOutputMessageTransport(
        object=data.get('object'),
        type=data.get('type'),
        mcp=mapManagementInstanceSessionsEventsGetOutputMessageTransportMcp.from_dict(data.get('mcp')) if data.get('mcp') else None,
        tool_call=mapManagementInstanceSessionsEventsGetOutputMessageTransportToolCall.from_dict(data.get('tool_call')) if data.get('tool_call') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageTransport, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags(
        destructive=data.get('destructive'),
        read_only=data.get('read_only')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCallTool:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCallTool:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCallTool(
        object=data.get('object'),
        id=data.get('id'),
        key=data.get('key'),
        name=data.get('name'),
        description=data.get('description'),
        capabilities=data.get('capabilities'),
        constraints=data.get('constraints', []),
        instructions=data.get('instructions', []),
        input_schema=mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolInputSchema.from_dict(data.get('input_schema')) if data.get('input_schema') else None,
        output_schema=mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolOutputSchema.from_dict(data.get('output_schema')) if data.get('output_schema') else None,
        tags=mapManagementInstanceSessionsEventsGetOutputMessageToolCallToolTags.from_dict(data.get('tags')) if data.get('tags') else None,
        specification_id=data.get('specification_id'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCallTool, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCallError:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCallError:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCallError(
        object=data.get('object'),
        id=data.get('id'),
        code=data.get('code'),
        message=data.get('message'),
        data=data.get('data'),
        status=data.get('status'),
        session_id=data.get('session_id'),
        provider_run_id=data.get('provider_run_id'),
        connection_id=data.get('connection_id'),
        group_id=data.get('group_id'),
        similar_error_count=data.get('similar_error_count'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCallError, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageToolCall:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageToolCall:
        return ManagementInstanceSessionsEventsGetOutputMessageToolCall(
        object=data.get('object'),
        id=data.get('id'),
        tool_key=data.get('tool_key'),
        type=data.get('type'),
        status=data.get('status'),
        source=data.get('source'),
        transport=data.get('transport'),
        session_id=data.get('session_id'),
        message_id=data.get('message_id'),
        session_provider_id=data.get('session_provider_id'),
        connection_id=data.get('connection_id'),
        provider_run_id=data.get('provider_run_id'),
        tool=mapManagementInstanceSessionsEventsGetOutputMessageToolCallTool.from_dict(data.get('tool')) if data.get('tool') else None,
        error=mapManagementInstanceSessionsEventsGetOutputMessageToolCallError.from_dict(data.get('error')) if data.get('error') else None,
        input=data.get('input'),
        output=data.get('output'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageToolCall, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData:
        return ManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData(
        identifier=data.get('identifier'),
        name=data.get('name')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageSenderParticipant:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageSenderParticipant:
        return ManagementInstanceSessionsEventsGetOutputMessageSenderParticipant(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        data=mapManagementInstanceSessionsEventsGetOutputMessageSenderParticipantData.from_dict(data.get('data')) if data.get('data') else None,
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageSenderParticipant, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData:
        return ManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData(
        identifier=data.get('identifier'),
        name=data.get('name')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageResponderParticipant:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageResponderParticipant:
        return ManagementInstanceSessionsEventsGetOutputMessageResponderParticipant(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        data=mapManagementInstanceSessionsEventsGetOutputMessageResponderParticipantData.from_dict(data.get('data')) if data.get('data') else None,
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageResponderParticipant, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessageError:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessageError:
        return ManagementInstanceSessionsEventsGetOutputMessageError(
        object=data.get('object'),
        id=data.get('id'),
        code=data.get('code'),
        message=data.get('message'),
        data=data.get('data'),
        status=data.get('status'),
        session_id=data.get('session_id'),
        provider_run_id=data.get('provider_run_id'),
        connection_id=data.get('connection_id'),
        group_id=data.get('group_id'),
        similar_error_count=data.get('similar_error_count'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessageError, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputMessage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputMessage:
        return ManagementInstanceSessionsEventsGetOutputMessage(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        status=data.get('status'),
        source=data.get('source'),
        session_id=data.get('session_id'),
        session_provider_id=data.get('session_provider_id'),
        connection_id=data.get('connection_id'),
        provider_run_id=data.get('provider_run_id'),
        hierarchy=mapManagementInstanceSessionsEventsGetOutputMessageHierarchy.from_dict(data.get('hierarchy')) if data.get('hierarchy') else None,
        transport=mapManagementInstanceSessionsEventsGetOutputMessageTransport.from_dict(data.get('transport')) if data.get('transport') else None,
        input=data.get('input'),
        output=data.get('output'),
        tool_call=mapManagementInstanceSessionsEventsGetOutputMessageToolCall.from_dict(data.get('tool_call')) if data.get('tool_call') else None,
        sender_participant=mapManagementInstanceSessionsEventsGetOutputMessageSenderParticipant.from_dict(data.get('sender_participant')) if data.get('sender_participant') else None,
        responder_participant=mapManagementInstanceSessionsEventsGetOutputMessageResponderParticipant.from_dict(data.get('responder_participant')) if data.get('responder_participant') else None,
        error=mapManagementInstanceSessionsEventsGetOutputMessageError.from_dict(data.get('error')) if data.get('error') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputMessage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputError:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputError:
        return ManagementInstanceSessionsEventsGetOutputError(
        object=data.get('object'),
        id=data.get('id'),
        code=data.get('code'),
        message=data.get('message'),
        data=data.get('data'),
        status=data.get('status'),
        session_id=data.get('session_id'),
        provider_run_id=data.get('provider_run_id'),
        connection_id=data.get('connection_id'),
        group_id=data.get('group_id'),
        similar_error_count=data.get('similar_error_count'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputError, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutputWarning:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutputWarning:
        return ManagementInstanceSessionsEventsGetOutputWarning(
        object=data.get('object'),
        id=data.get('id'),
        code=data.get('code'),
        message=data.get('message'),
        data=data.get('data'),
        session_id=data.get('session_id'),
        connection_id=data.get('connection_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutputWarning, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsEventsGetOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsEventsGetOutput:
        return ManagementInstanceSessionsEventsGetOutput(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        session_id=data.get('session_id'),
        connection=mapManagementInstanceSessionsEventsGetOutputConnection.from_dict(data.get('connection')) if data.get('connection') else None,
        provider_run=mapManagementInstanceSessionsEventsGetOutputProviderRun.from_dict(data.get('provider_run')) if data.get('provider_run') else None,
        message=mapManagementInstanceSessionsEventsGetOutputMessage.from_dict(data.get('message')) if data.get('message') else None,
        error=mapManagementInstanceSessionsEventsGetOutputError.from_dict(data.get('error')) if data.get('error') else None,
        warning=mapManagementInstanceSessionsEventsGetOutputWarning.from_dict(data.get('warning')) if data.get('warning') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsEventsGetOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ProviderDeploymentsUpdateOutputLockedVersion:
    object: str
    id: str
    version: str
    provider_id: str
    is_current: bool
    name: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    specification_id: Optional[str] = None
@dataclass
class ProviderDeploymentsUpdateOutputDefaultConfig:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ProviderDeploymentsUpdateOutput:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    locked_version: Optional[ProviderDeploymentsUpdateOutputLockedVersion] = None
    default_config: Optional[ProviderDeploymentsUpdateOutputDefaultConfig] = None


class mapProviderDeploymentsUpdateOutputLockedVersion:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsUpdateOutputLockedVersion:
        return ProviderDeploymentsUpdateOutputLockedVersion(
        object=data.get('object'),
        id=data.get('id'),
        version=data.get('version'),
        provider_id=data.get('provider_id'),
        is_current=data.get('is_current'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        specification_id=data.get('specification_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsUpdateOutputLockedVersion, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsUpdateOutputDefaultConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsUpdateOutputDefaultConfig:
        return ProviderDeploymentsUpdateOutputDefaultConfig(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsUpdateOutputDefaultConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsUpdateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsUpdateOutput:
        return ProviderDeploymentsUpdateOutput(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        locked_version=mapProviderDeploymentsUpdateOutputLockedVersion.from_dict(data.get('locked_version')) if data.get('locked_version') else None,
        default_config=mapProviderDeploymentsUpdateOutputDefaultConfig.from_dict(data.get('default_config')) if data.get('default_config') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsUpdateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ProviderDeploymentsUpdateBody:
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None


class mapProviderDeploymentsUpdateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsUpdateBody:
        return ProviderDeploymentsUpdateBody(
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsUpdateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ProviderOauthConnectionTemplateGetOutputProvider:
  name: str
  url: str
  image_url: str


@dataclass
class ProviderOauthConnectionTemplateGetOutputScopes:
  id: str
  identifier: str
  description: str


@dataclass
class ProviderOauthConnectionTemplateGetOutputVariables:
  id: str
  key: str
  type: str
  label: str
  description: Optional[str] = None


@dataclass
class ProviderOauthConnectionTemplateGetOutputProfileBadges:
  type: str
  name: str


@dataclass
class ProviderOauthConnectionTemplateGetOutputProfile:
  object: str
  id: str
  name: str
  slug: str
  image_url: str
  is_official: bool
  is_metorial: bool
  is_verified: bool
  badges: List[ProviderOauthConnectionTemplateGetOutputProfileBadges]
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ProviderOauthConnectionTemplateGetOutput:
  object: str
  id: str
  status: str
  slug: str
  name: str
  provider: ProviderOauthConnectionTemplateGetOutputProvider
  scopes: List[ProviderOauthConnectionTemplateGetOutputScopes]
  variables: List[ProviderOauthConnectionTemplateGetOutputVariables]
  profile: ProviderOauthConnectionTemplateGetOutputProfile
  created_at: datetime
  updated_at: datetime


class mapProviderOauthConnectionTemplateGetOutputProvider:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ProviderOauthConnectionTemplateGetOutputProvider:
    return ProviderOauthConnectionTemplateGetOutputProvider(
      name=data.get("name"), url=data.get("url"), image_url=data.get("image_url")
    )

  @staticmethod
  def to_dict(
    value: Union[ProviderOauthConnectionTemplateGetOutputProvider, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapProviderOauthConnectionTemplateGetOutputScopes:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ProviderOauthConnectionTemplateGetOutputScopes:
    return ProviderOauthConnectionTemplateGetOutputScopes(
      id=data.get("id"),
      identifier=data.get("identifier"),
      description=data.get("description"),
    )

  @staticmethod
  def to_dict(
    value: Union[ProviderOauthConnectionTemplateGetOutputScopes, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapProviderOauthConnectionTemplateGetOutputVariables:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ProviderOauthConnectionTemplateGetOutputVariables:
    return ProviderOauthConnectionTemplateGetOutputVariables(
      id=data.get("id"),
      key=data.get("key"),
      type=data.get("type"),
      label=data.get("label"),
      description=data.get("description"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ProviderOauthConnectionTemplateGetOutputVariables, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapProviderOauthConnectionTemplateGetOutputProfileBadges:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ProviderOauthConnectionTemplateGetOutputProfileBadges:
    return ProviderOauthConnectionTemplateGetOutputProfileBadges(
      type=data.get("type"), name=data.get("name")
    )

  @staticmethod
  def to_dict(
    value: Union[
      ProviderOauthConnectionTemplateGetOutputProfileBadges, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapProviderOauthConnectionTemplateGetOutputProfile:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ProviderOauthConnectionTemplateGetOutputProfile:
    return ProviderOauthConnectionTemplateGetOutputProfile(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      slug=data.get("slug"),
      image_url=data.get("image_url"),
      is_official=data.get("is_official"),
      is_metorial=data.get("is_metorial"),
      is_verified=data.get("is_verified"),
      badges=[
        mapProviderOauthConnectionTemplateGetOutputProfileBadges.from_dict(item)
        for item in data.get("badges", [])
        if item
      ],
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ProviderOauthConnectionTemplateGetOutputProfile, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapProviderOauthConnectionTemplateGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ProviderOauthConnectionTemplateGetOutput:
    return ProviderOauthConnectionTemplateGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      slug=data.get("slug"),
      name=data.get("name"),
      provider=mapProviderOauthConnectionTemplateGetOutputProvider.from_dict(
        data.get("provider")
      )
      if data.get("provider")
      else None,
      scopes=[
        mapProviderOauthConnectionTemplateGetOutputScopes.from_dict(item)
        for item in data.get("scopes", [])
        if item
      ],
      variables=[
        mapProviderOauthConnectionTemplateGetOutputVariables.from_dict(item)
        for item in data.get("variables", [])
        if item
      ],
      profile=mapProviderOauthConnectionTemplateGetOutputProfile.from_dict(
        data.get("profile")
      )
      if data.get("profile")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ProviderOauthConnectionTemplateGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapProviderOauthDiscoverOutput,
  ProviderOauthDiscoverOutput,
  mapProviderOauthDiscoverBody,
  ProviderOauthDiscoverBody,
)


class MetorialProviderOauthEndpoint(BaseMetorialEndpoint):
  """Get OAuth connection template information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def discover(
    self, *, discovery_url: str, client_name: str
  ) -> ProviderOauthDiscoverOutput:
    """
    Discover OAuth Configuration
    Discover OAuth configuration from a discovery URL

    :param discovery_url: str
    :param client_name: str
    :return: ProviderOauthDiscoverOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    body_dict["discovery_url"] = discovery_url
    body_dict["client_name"] = client_name

    request = MetorialRequest(path=["provider-oauth-discovery"], body=body_dict)
    return self._post(request).transform(mapProviderOauthDiscoverOutput.from_dict)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ManagementInstanceCallbacksListOutputItemsProviderDeployment:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceCallbacksListOutputItemsDestinations:
    object: str
    id: str
    status: str
    name: str
    url: str
    method: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger:
    object: str
    id: str
    key: str
    name: str
@dataclass
class ManagementInstanceCallbacksListOutputItemsProviderTriggers:
    object: str
    id: str
    provider_trigger: ManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger
    event_types: List[str]
    created_at: datetime
@dataclass
class ManagementInstanceCallbacksListOutputItems:
    object: str
    id: str
    status: str
    name: str
    provider_deployment: ManagementInstanceCallbacksListOutputItemsProviderDeployment
    destinations: List[ManagementInstanceCallbacksListOutputItemsDestinations]
    provider_triggers: List[ManagementInstanceCallbacksListOutputItemsProviderTriggers]
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    poll_interval_seconds_override: Optional[float] = None
@dataclass
class ManagementInstanceCallbacksListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class ManagementInstanceCallbacksListOutput:
    items: List[ManagementInstanceCallbacksListOutputItems]
    pagination: ManagementInstanceCallbacksListOutputPagination


class mapManagementInstanceCallbacksListOutputItemsProviderDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputItemsProviderDeployment:
        return ManagementInstanceCallbacksListOutputItemsProviderDeployment(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputItemsProviderDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutputItemsDestinations:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputItemsDestinations:
        return ManagementInstanceCallbacksListOutputItemsDestinations(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        url=data.get('url'),
        method=data.get('method'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputItemsDestinations, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger:
        return ManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger(
        object=data.get('object'),
        id=data.get('id'),
        key=data.get('key'),
        name=data.get('name')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutputItemsProviderTriggers:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputItemsProviderTriggers:
        return ManagementInstanceCallbacksListOutputItemsProviderTriggers(
        object=data.get('object'),
        id=data.get('id'),
        provider_trigger=mapManagementInstanceCallbacksListOutputItemsProviderTriggersProviderTrigger.from_dict(data.get('provider_trigger')) if data.get('provider_trigger') else None,
        event_types=data.get('event_types', []),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputItemsProviderTriggers, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputItems:
        return ManagementInstanceCallbacksListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        poll_interval_seconds_override=data.get('poll_interval_seconds_override'),
        provider_deployment=mapManagementInstanceCallbacksListOutputItemsProviderDeployment.from_dict(data.get('provider_deployment')) if data.get('provider_deployment') else None,
        destinations=[mapManagementInstanceCallbacksListOutputItemsDestinations.from_dict(item) for item in data.get('destinations', []) if item],
        provider_triggers=[mapManagementInstanceCallbacksListOutputItemsProviderTriggers.from_dict(item) for item in data.get('provider_triggers', []) if item],
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutputPagination:
        return ManagementInstanceCallbacksListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListOutput:
        return ManagementInstanceCallbacksListOutput(
        items=[mapManagementInstanceCallbacksListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapManagementInstanceCallbacksListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ManagementInstanceCallbacksListQueryCreatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class ManagementInstanceCallbacksListQueryUpdatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class ManagementInstanceCallbacksListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    id: Optional[Union[str, List[str]]] = None
    provider_deployment_id: Optional[Union[str, List[str]]] = None
    status: Optional[Union[str, List[str]]] = None
    created_at: Optional[ManagementInstanceCallbacksListQueryCreatedAt] = None
    updated_at: Optional[ManagementInstanceCallbacksListQueryUpdatedAt] = None


class mapManagementInstanceCallbacksListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksListQuery:
        return ManagementInstanceCallbacksListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        id=data.get('id'),
        provider_deployment_id=data.get('provider_deployment_id'),
        status=data.get('status'),
        created_at=mapManagementInstanceCallbacksListQueryCreatedAt.from_dict(data.get('created_at')) if data.get('created_at') else None,
        updated_at=mapManagementInstanceCallbacksListQueryUpdatedAt.from_dict(data.get('updated_at')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class CustomProvidersGetOutputDraftContainerImage:
    object: str
    container_registry: str
    container_image_tag: str
    container_image: str
@dataclass
class CustomProvidersGetOutputDraftRemoteMcpServer:
    object: str
    url: str
    transport: str
@dataclass
class CustomProvidersGetOutputDraftConfigSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class CustomProvidersGetOutputDraftConfig:
    object: str
    schema: CustomProvidersGetOutputDraftConfigSchema
    transformer: str
@dataclass
class CustomProvidersGetOutputDraft:
    object: str
    config: CustomProvidersGetOutputDraftConfig
    container_image: Optional[CustomProvidersGetOutputDraftContainerImage] = None
    remote_mcp_server: Optional[CustomProvidersGetOutputDraftRemoteMcpServer] = None
@dataclass
class CustomProvidersGetOutputScmRepoProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class CustomProvidersGetOutputScmRepo:
    object: str
    id: str
    provider: CustomProvidersGetOutputScmRepoProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class CustomProvidersGetOutputProviderPublisher:
    object: str
    id: str
    name: str
    image_url: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
@dataclass
class CustomProvidersGetOutputProviderCurrentVersion:
    object: str
    id: str
    version: str
    provider_id: str
    is_current: bool
    name: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    specification_id: Optional[str] = None
@dataclass
class CustomProvidersGetOutputProviderOauthAutoRegistration:
    status: str
@dataclass
class CustomProvidersGetOutputProviderOauth:
    status: str
    auto_registration: CustomProvidersGetOutputProviderOauthAutoRegistration
    callback_url: Optional[str] = None
@dataclass
class CustomProvidersGetOutputProvider:
    object: str
    id: str
    access: str
    status: str
    publisher: CustomProvidersGetOutputProviderPublisher
    identifier: str
    name: str
    slug: str
    created_at: datetime
    updated_at: datetime
    current_version: Optional[CustomProvidersGetOutputProviderCurrentVersion] = None
    oauth: Optional[CustomProvidersGetOutputProviderOauth] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class CustomProvidersGetOutput:
    object: str
    id: str
    status: str
    type: str
    name: str
    draft: CustomProvidersGetOutputDraft
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    scm_repo: Optional[CustomProvidersGetOutputScmRepo] = None
    provider: Optional[CustomProvidersGetOutputProvider] = None


class mapCustomProvidersGetOutputDraftContainerImage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputDraftContainerImage:
        return CustomProvidersGetOutputDraftContainerImage(
        object=data.get('object'),
        container_registry=data.get('container_registry'),
        container_image_tag=data.get('container_image_tag'),
        container_image=data.get('container_image')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputDraftContainerImage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputDraftRemoteMcpServer:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputDraftRemoteMcpServer:
        return CustomProvidersGetOutputDraftRemoteMcpServer(
        object=data.get('object'),
        url=data.get('url'),
        transport=data.get('transport')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputDraftRemoteMcpServer, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputDraftConfigSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputDraftConfigSchema:
        return CustomProvidersGetOutputDraftConfigSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputDraftConfigSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputDraftConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputDraftConfig:
        return CustomProvidersGetOutputDraftConfig(
        object=data.get('object'),
        schema=mapCustomProvidersGetOutputDraftConfigSchema.from_dict(data.get('schema')) if data.get('schema') else None,
        transformer=data.get('transformer')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputDraftConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputDraft:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputDraft:
        return CustomProvidersGetOutputDraft(
        object=data.get('object'),
        container_image=mapCustomProvidersGetOutputDraftContainerImage.from_dict(data.get('container_image')) if data.get('container_image') else None,
        remote_mcp_server=mapCustomProvidersGetOutputDraftRemoteMcpServer.from_dict(data.get('remote_mcp_server')) if data.get('remote_mcp_server') else None,
        config=mapCustomProvidersGetOutputDraftConfig.from_dict(data.get('config')) if data.get('config') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputDraft, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputScmRepoProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputScmRepoProvider:
        return CustomProvidersGetOutputScmRepoProvider(
        object=data.get('object'),
        type=data.get('type'),
        id=data.get('id'),
        name=data.get('name'),
        owner=data.get('owner')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputScmRepoProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputScmRepo:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputScmRepo:
        return CustomProvidersGetOutputScmRepo(
        object=data.get('object'),
        id=data.get('id'),
        provider=mapCustomProvidersGetOutputScmRepoProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        url=data.get('url'),
        is_private=data.get('is_private'),
        default_branch=data.get('default_branch'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputScmRepo, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputProviderPublisher:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputProviderPublisher:
        return CustomProvidersGetOutputProviderPublisher(
        object=data.get('object'),
        id=data.get('id'),
        name=data.get('name'),
        description=data.get('description'),
        image_url=data.get('image_url'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputProviderPublisher, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputProviderCurrentVersion:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputProviderCurrentVersion:
        return CustomProvidersGetOutputProviderCurrentVersion(
        object=data.get('object'),
        id=data.get('id'),
        version=data.get('version'),
        provider_id=data.get('provider_id'),
        is_current=data.get('is_current'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        specification_id=data.get('specification_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputProviderCurrentVersion, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputProviderOauthAutoRegistration:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputProviderOauthAutoRegistration:
        return CustomProvidersGetOutputProviderOauthAutoRegistration(
        status=data.get('status')
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputProviderOauthAutoRegistration, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputProviderOauth:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputProviderOauth:
        return CustomProvidersGetOutputProviderOauth(
        status=data.get('status'),
        callback_url=data.get('callback_url'),
        auto_registration=mapCustomProvidersGetOutputProviderOauthAutoRegistration.from_dict(data.get('auto_registration')) if data.get('auto_registration') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputProviderOauth, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutputProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutputProvider:
        return CustomProvidersGetOutputProvider(
        object=data.get('object'),
        id=data.get('id'),
        access=data.get('access'),
        status=data.get('status'),
        publisher=mapCustomProvidersGetOutputProviderPublisher.from_dict(data.get('publisher')) if data.get('publisher') else None,
        current_version=mapCustomProvidersGetOutputProviderCurrentVersion.from_dict(data.get('current_version')) if data.get('current_version') else None,
        oauth=mapCustomProvidersGetOutputProviderOauth.from_dict(data.get('oauth')) if data.get('oauth') else None,
        identifier=data.get('identifier'),
        name=data.get('name'),
        description=data.get('description'),
        slug=data.get('slug'),
        metadata=data.get('metadata'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutputProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapCustomProvidersGetOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> CustomProvidersGetOutput:
        return CustomProvidersGetOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        type=data.get('type'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        draft=mapCustomProvidersGetOutputDraft.from_dict(data.get('draft')) if data.get('draft') else None,
        scm_repo=mapCustomProvidersGetOutputScmRepo.from_dict(data.get('scm_repo')) if data.get('scm_repo') else None,
        provider=mapCustomProvidersGetOutputProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[CustomProvidersGetOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsMcpClient:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsMcpServer:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsMcp:
  object: str
  version: str
  connection_type: str
  client: Optional[
    DashboardInstanceSessionsServerSessionsListOutputItemsMcpClient
  ] = None
  server: Optional[
    DashboardInstanceSessionsServerSessionsListOutputItemsMcpServer
  ] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsSession:
  object: str
  id: str
  status: str
  connection_status: str
  usage: DashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: DashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient:
  user_agent: str
  anonymized_ip_address: str


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItemsConnection:
  object: str
  id: str
  client: DashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient
  created_at: datetime
  started_at: datetime
  ended_at: Optional[datetime] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputItems:
  object: str
  id: str
  status: str
  mcp: DashboardInstanceSessionsServerSessionsListOutputItemsMcp
  usage: DashboardInstanceSessionsServerSessionsListOutputItemsUsage
  server: DashboardInstanceSessionsServerSessionsListOutputItemsServer
  session: DashboardInstanceSessionsServerSessionsListOutputItemsSession
  server_deployment: DashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment
  created_at: datetime
  connection: Optional[
    DashboardInstanceSessionsServerSessionsListOutputItemsConnection
  ] = None


@dataclass
class DashboardInstanceSessionsServerSessionsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class DashboardInstanceSessionsServerSessionsListOutput:
  items: List[DashboardInstanceSessionsServerSessionsListOutputItems]
  pagination: DashboardInstanceSessionsServerSessionsListOutputPagination


class mapDashboardInstanceSessionsServerSessionsListOutputItemsMcpClient:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsMcpClient:
    return DashboardInstanceSessionsServerSessionsListOutputItemsMcpClient(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsMcpClient,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsMcpServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsMcpServer:
    return DashboardInstanceSessionsServerSessionsListOutputItemsMcpServer(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsMcpServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsMcp:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsMcp:
    return DashboardInstanceSessionsServerSessionsListOutputItemsMcp(
      object=data.get("object"),
      version=data.get("version"),
      connection_type=data.get("connection_type"),
      client=mapDashboardInstanceSessionsServerSessionsListOutputItemsMcpClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      server=mapDashboardInstanceSessionsServerSessionsListOutputItemsMcpServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsMcp, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsUsage:
    return DashboardInstanceSessionsServerSessionsListOutputItemsUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsUsage, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsServer:
    return DashboardInstanceSessionsServerSessionsListOutputItemsServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage:
    return DashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsSession:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsSession:
    return DashboardInstanceSessionsServerSessionsListOutputItemsSession(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      connection_status=data.get("connection_status"),
      usage=mapDashboardInstanceSessionsServerSessionsListOutputItemsSessionUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsSession,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
    return DashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment:
    return DashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapDashboardInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient:
    return DashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient(
      user_agent=data.get("user_agent"),
      anonymized_ip_address=data.get("anonymized_ip_address"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItemsConnection:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItemsConnection:
    return DashboardInstanceSessionsServerSessionsListOutputItemsConnection(
      object=data.get("object"),
      id=data.get("id"),
      client=mapDashboardInstanceSessionsServerSessionsListOutputItemsConnectionClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      started_at=parse_iso_datetime(data.get("started_at"))
      if data.get("started_at")
      else None,
      ended_at=parse_iso_datetime(data.get("ended_at"))
      if data.get("ended_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItemsConnection,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputItems:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputItems:
    return DashboardInstanceSessionsServerSessionsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      mcp=mapDashboardInstanceSessionsServerSessionsListOutputItemsMcp.from_dict(
        data.get("mcp")
      )
      if data.get("mcp")
      else None,
      usage=mapDashboardInstanceSessionsServerSessionsListOutputItemsUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      server=mapDashboardInstanceSessionsServerSessionsListOutputItemsServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      session=mapDashboardInstanceSessionsServerSessionsListOutputItemsSession.from_dict(
        data.get("session")
      )
      if data.get("session")
      else None,
      server_deployment=mapDashboardInstanceSessionsServerSessionsListOutputItemsServerDeployment.from_dict(
        data.get("server_deployment")
      )
      if data.get("server_deployment")
      else None,
      connection=mapDashboardInstanceSessionsServerSessionsListOutputItemsConnection.from_dict(
        data.get("connection")
      )
      if data.get("connection")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputItems, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutputPagination:
    return DashboardInstanceSessionsServerSessionsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutputPagination, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsServerSessionsListOutput:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListOutput:
    return DashboardInstanceSessionsServerSessionsListOutput(
      items=[
        mapDashboardInstanceSessionsServerSessionsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapDashboardInstanceSessionsServerSessionsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsServerSessionsListOutput, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardInstanceSessionsServerSessionsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None


class mapDashboardInstanceSessionsServerSessionsListQuery:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsServerSessionsListQuery:
    return DashboardInstanceSessionsServerSessionsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceSessionsServerSessionsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from .resources import *
from .endpoints import *

# Type exports for better discoverability
# Generated type exports
# These types are automatically exported for better IDE support

# Type classes
# DashboardInstanceCallbacksDestinationsCreateBody
# DashboardInstanceCallbacksDestinationsCreateOutput
# DashboardInstanceCallbacksDestinationsDeleteOutput
# DashboardInstanceCallbacksDestinationsGetOutput
# DashboardInstanceCallbacksDestinationsListOutput
# DashboardInstanceCallbacksDestinationsListQuery
# DashboardInstanceCallbacksDestinationsUpdateBody
# DashboardInstanceCallbacksDestinationsUpdateOutput
# DashboardInstanceCallbacksEventsGetOutput
# DashboardInstanceCallbacksEventsListOutput
# DashboardInstanceCallbacksEventsListQuery
# DashboardInstanceCallbacksGetOutput
# DashboardInstanceCallbacksListOutput
# DashboardInstanceCallbacksListQuery
# DashboardInstanceCallbacksNotificationsGetOutput
# DashboardInstanceCallbacksNotificationsListOutput
# DashboardInstanceCallbacksNotificationsListQuery
# DashboardInstanceFilesDeleteOutput
# DashboardInstanceFilesGetOutput
# DashboardInstanceFilesListOutput
# DashboardInstanceFilesListQuery
# DashboardInstanceFilesUpdateBody
# DashboardInstanceFilesUpdateOutput
# DashboardInstanceInstanceGetOutput
# DashboardInstanceLinksCreateBody
# DashboardInstanceLinksCreateOutput
# DashboardInstanceLinksDeleteOutput
# DashboardInstanceLinksGetOutput
# DashboardInstanceLinksListOutput
# DashboardInstanceLinksUpdateBody
# DashboardInstanceLinksUpdateOutput
# DashboardInstanceMagicMcpServersCreateBody
# DashboardInstanceMagicMcpServersCreateOutput
# DashboardInstanceMagicMcpServersDeleteOutput
# DashboardInstanceMagicMcpServersGetOutput
# DashboardInstanceMagicMcpServersListOutput
# DashboardInstanceMagicMcpServersListQuery
# DashboardInstanceMagicMcpServersUpdateBody
# DashboardInstanceMagicMcpServersUpdateOutput
# DashboardInstanceMagicMcpSessionsGetOutput
# DashboardInstanceMagicMcpSessionsListOutput
# DashboardInstanceMagicMcpSessionsListQuery
# DashboardInstanceMagicMcpTokensCreateBody
# DashboardInstanceMagicMcpTokensCreateOutput
# DashboardInstanceMagicMcpTokensDeleteOutput
# DashboardInstanceMagicMcpTokensGetOutput
# DashboardInstanceMagicMcpTokensListOutput
# DashboardInstanceMagicMcpTokensListQuery
# DashboardInstanceMagicMcpTokensUpdateBody
# DashboardInstanceMagicMcpTokensUpdateOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsGetOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsListOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsListQuery
# DashboardInstanceProviderOauthConnectionsCreateBody
# DashboardInstanceProviderOauthConnectionsCreateOutput
# DashboardInstanceProviderOauthConnectionsDeleteOutput
# DashboardInstanceProviderOauthConnectionsGetOutput
# DashboardInstanceProviderOauthConnectionsListOutput
# DashboardInstanceProviderOauthConnectionsListQuery
# DashboardInstanceProviderOauthConnectionsProfilesGetOutput
# DashboardInstanceProviderOauthConnectionsProfilesListOutput
# DashboardInstanceProviderOauthConnectionsProfilesListQuery
# DashboardInstanceProviderOauthConnectionsUpdateBody
# DashboardInstanceProviderOauthConnectionsUpdateOutput
# DashboardInstanceProviderOauthSessionsCreateBody
# DashboardInstanceProviderOauthSessionsCreateOutput
# DashboardInstanceProviderOauthSessionsDeleteOutput
# DashboardInstanceProviderOauthSessionsGetOutput
# DashboardInstanceProviderOauthSessionsListOutput
# DashboardInstanceProviderOauthSessionsListQuery
# DashboardInstanceProviderOauthTakeoutsCreateBody
# DashboardInstanceProviderOauthTakeoutsCreateOutput
# DashboardInstanceProviderOauthTakeoutsGetOutput
# DashboardInstanceProviderOauthTakeoutsListOutput
# DashboardInstanceProviderOauthTakeoutsListQuery
# DashboardInstanceSecretsGetOutput
# DashboardInstanceSecretsListOutput
# DashboardInstanceSecretsListQuery
# DashboardInstanceServerConfigVaultsCreateBody
# DashboardInstanceServerConfigVaultsCreateOutput
# DashboardInstanceServerConfigVaultsGetOutput
# DashboardInstanceServerConfigVaultsListOutput
# DashboardInstanceServerConfigVaultsListQuery
# DashboardInstanceServerConfigVaultsUpdateBody
# DashboardInstanceServerConfigVaultsUpdateOutput
# DashboardInstanceServerRunErrorsGetOutput
# DashboardInstanceServerRunErrorsListOutput
# DashboardInstanceServerRunErrorsListQuery
# DashboardInstanceServerRunsGetOutput
# DashboardInstanceServerRunsListOutput
# DashboardInstanceServerRunsListQuery
# DashboardInstanceServersCapabilitiesListOutput
# DashboardInstanceServersCapabilitiesListQuery
# DashboardInstanceServersDeploymentsCreateBody
# DashboardInstanceServersDeploymentsCreateOutput
# DashboardInstanceServersDeploymentsDeleteOutput
# DashboardInstanceServersDeploymentsGetOutput
# DashboardInstanceServersDeploymentsListOutput
# DashboardInstanceServersDeploymentsListQuery
# DashboardInstanceServersDeploymentsUpdateBody
# DashboardInstanceServersDeploymentsUpdateOutput
# DashboardInstanceServersGetOutput
# DashboardInstanceServersImplementationsCreateBody
# DashboardInstanceServersImplementationsCreateOutput
# DashboardInstanceServersImplementationsDeleteOutput
# DashboardInstanceServersImplementationsGetOutput
# DashboardInstanceServersImplementationsListOutput
# DashboardInstanceServersImplementationsListQuery
# DashboardInstanceServersImplementationsUpdateBody
# DashboardInstanceServersImplementationsUpdateOutput
# DashboardInstanceServersVariantsGetOutput
# DashboardInstanceServersVariantsListOutput
# DashboardInstanceServersVariantsListQuery
# DashboardInstanceServersVersionsGetOutput
# DashboardInstanceServersVersionsListOutput
# DashboardInstanceServersVersionsListQuery
# DashboardInstanceSessionsConnectionsGetOutput
# DashboardInstanceSessionsConnectionsListOutput
# DashboardInstanceSessionsConnectionsListQuery
# DashboardInstanceSessionsCreateBody
# DashboardInstanceSessionsCreateOutput
# DashboardInstanceSessionsDeleteOutput
# DashboardInstanceSessionsGetOutput
# DashboardInstanceSessionsListOutput
# DashboardInstanceSessionsListQuery
# DashboardInstanceSessionsMessagesGetOutput
# DashboardInstanceSessionsMessagesListOutput
# DashboardInstanceSessionsMessagesListQuery
# DashboardOrganizationsTeamsCreateBody
# DashboardOrganizationsTeamsCreateOutput
# DashboardOrganizationsTeamsGetOutput
# DashboardOrganizationsTeamsListOutput
# DashboardOrganizationsTeamsListQuery
# DashboardOrganizationsTeamsMembersCreateBody
# DashboardOrganizationsTeamsMembersCreateOutput
# DashboardOrganizationsTeamsMembersDeleteOutput
# DashboardOrganizationsTeamsPermissionsOutput
# DashboardOrganizationsTeamsProjectsRemoveOutput
# DashboardOrganizationsTeamsProjectsSetBody
# DashboardOrganizationsTeamsProjectsSetOutput
# DashboardOrganizationsTeamsRolesCreateBody
# DashboardOrganizationsTeamsRolesCreateOutput
# DashboardOrganizationsTeamsRolesGetOutput
# DashboardOrganizationsTeamsRolesListOutput
# DashboardOrganizationsTeamsRolesListQuery
# DashboardOrganizationsTeamsRolesUpdateBody
# DashboardOrganizationsTeamsRolesUpdateOutput
# DashboardOrganizationsTeamsUpdateBody
# DashboardOrganizationsTeamsUpdateOutput

# Mapper classes
# mapDashboardInstanceCallbacksDestinationsCreateBody
# mapDashboardInstanceCallbacksDestinationsCreateOutput
# mapDashboardInstanceCallbacksDestinationsDeleteOutput
# mapDashboardInstanceCallbacksDestinationsGetOutput
# mapDashboardInstanceCallbacksDestinationsListOutput
# mapDashboardInstanceCallbacksDestinationsListQuery
# mapDashboardInstanceCallbacksDestinationsUpdateBody
# mapDashboardInstanceCallbacksDestinationsUpdateOutput
# mapDashboardInstanceCallbacksEventsGetOutput
# mapDashboardInstanceCallbacksEventsListOutput
# mapDashboardInstanceCallbacksEventsListQuery
# mapDashboardInstanceCallbacksGetOutput
# mapDashboardInstanceCallbacksListOutput
# mapDashboardInstanceCallbacksListQuery
# mapDashboardInstanceCallbacksNotificationsGetOutput
# mapDashboardInstanceCallbacksNotificationsListOutput
# mapDashboardInstanceCallbacksNotificationsListQuery
# mapDashboardInstanceFilesDeleteOutput
# mapDashboardInstanceFilesGetOutput
# mapDashboardInstanceFilesListOutput
# mapDashboardInstanceFilesListQuery
# mapDashboardInstanceFilesUpdateBody
# mapDashboardInstanceFilesUpdateOutput
# mapDashboardInstanceInstanceGetOutput
# mapDashboardInstanceLinksCreateBody
# mapDashboardInstanceLinksCreateOutput
# mapDashboardInstanceLinksDeleteOutput
# mapDashboardInstanceLinksGetOutput
# mapDashboardInstanceLinksListOutput
# mapDashboardInstanceLinksUpdateBody
# mapDashboardInstanceLinksUpdateOutput
# mapDashboardInstanceMagicMcpServersCreateBody
# mapDashboardInstanceMagicMcpServersCreateOutput
# mapDashboardInstanceMagicMcpServersDeleteOutput
# mapDashboardInstanceMagicMcpServersGetOutput
# mapDashboardInstanceMagicMcpServersListOutput
# mapDashboardInstanceMagicMcpServersListQuery
# mapDashboardInstanceMagicMcpServersUpdateBody
# mapDashboardInstanceMagicMcpServersUpdateOutput
# mapDashboardInstanceMagicMcpSessionsGetOutput
# mapDashboardInstanceMagicMcpSessionsListOutput
# mapDashboardInstanceMagicMcpSessionsListQuery
# mapDashboardInstanceMagicMcpTokensCreateBody
# mapDashboardInstanceMagicMcpTokensCreateOutput
# mapDashboardInstanceMagicMcpTokensDeleteOutput
# mapDashboardInstanceMagicMcpTokensGetOutput
# mapDashboardInstanceMagicMcpTokensListOutput
# mapDashboardInstanceMagicMcpTokensListQuery
# mapDashboardInstanceMagicMcpTokensUpdateBody
# mapDashboardInstanceMagicMcpTokensUpdateOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsGetOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsListOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsListQuery
# mapDashboardInstanceProviderOauthConnectionsCreateBody
# mapDashboardInstanceProviderOauthConnectionsCreateOutput
# mapDashboardInstanceProviderOauthConnectionsDeleteOutput
# mapDashboardInstanceProviderOauthConnectionsGetOutput
# mapDashboardInstanceProviderOauthConnectionsListOutput
# mapDashboardInstanceProviderOauthConnectionsListQuery
# mapDashboardInstanceProviderOauthConnectionsProfilesGetOutput
# mapDashboardInstanceProviderOauthConnectionsProfilesListOutput
# mapDashboardInstanceProviderOauthConnectionsProfilesListQuery
# mapDashboardInstanceProviderOauthConnectionsUpdateBody
# mapDashboardInstanceProviderOauthConnectionsUpdateOutput
# mapDashboardInstanceProviderOauthSessionsCreateBody
# mapDashboardInstanceProviderOauthSessionsCreateOutput
# mapDashboardInstanceProviderOauthSessionsDeleteOutput
# mapDashboardInstanceProviderOauthSessionsGetOutput
# mapDashboardInstanceProviderOauthSessionsListOutput
# mapDashboardInstanceProviderOauthSessionsListQuery
# mapDashboardInstanceProviderOauthTakeoutsCreateBody
# mapDashboardInstanceProviderOauthTakeoutsCreateOutput
# mapDashboardInstanceProviderOauthTakeoutsGetOutput
# mapDashboardInstanceProviderOauthTakeoutsListOutput
# mapDashboardInstanceProviderOauthTakeoutsListQuery
# mapDashboardInstanceSecretsGetOutput
# mapDashboardInstanceSecretsListOutput
# mapDashboardInstanceSecretsListQuery
# mapDashboardInstanceServerConfigVaultsCreateBody
# mapDashboardInstanceServerConfigVaultsCreateOutput
# mapDashboardInstanceServerConfigVaultsGetOutput
# mapDashboardInstanceServerConfigVaultsListOutput
# mapDashboardInstanceServerConfigVaultsListQuery
# mapDashboardInstanceServerConfigVaultsUpdateBody
# mapDashboardInstanceServerConfigVaultsUpdateOutput
# mapDashboardInstanceServerRunErrorsGetOutput
# mapDashboardInstanceServerRunErrorsListOutput
# mapDashboardInstanceServerRunErrorsListQuery
# mapDashboardInstanceServerRunsGetOutput
# mapDashboardInstanceServerRunsListOutput
# mapDashboardInstanceServerRunsListQuery
# mapDashboardInstanceServersCapabilitiesListOutput
# mapDashboardInstanceServersCapabilitiesListQuery
# mapDashboardInstanceServersDeploymentsCreateBody
# mapDashboardInstanceServersDeploymentsCreateOutput
# mapDashboardInstanceServersDeploymentsDeleteOutput
# mapDashboardInstanceServersDeploymentsGetOutput
# mapDashboardInstanceServersDeploymentsListOutput
# mapDashboardInstanceServersDeploymentsListQuery
# mapDashboardInstanceServersDeploymentsUpdateBody
# mapDashboardInstanceServersDeploymentsUpdateOutput
# mapDashboardInstanceServersGetOutput
# mapDashboardInstanceServersImplementationsCreateBody
# mapDashboardInstanceServersImplementationsCreateOutput
# mapDashboardInstanceServersImplementationsDeleteOutput
# mapDashboardInstanceServersImplementationsGetOutput
# mapDashboardInstanceServersImplementationsListOutput
# mapDashboardInstanceServersImplementationsListQuery
# mapDashboardInstanceServersImplementationsUpdateBody
# mapDashboardInstanceServersImplementationsUpdateOutput
# mapDashboardInstanceServersVariantsGetOutput
# mapDashboardInstanceServersVariantsListOutput
# mapDashboardInstanceServersVariantsListQuery
# mapDashboardInstanceServersVersionsGetOutput
# mapDashboardInstanceServersVersionsListOutput
# mapDashboardInstanceServersVersionsListQuery
# mapDashboardInstanceSessionsConnectionsGetOutput
# mapDashboardInstanceSessionsConnectionsListOutput
# mapDashboardInstanceSessionsConnectionsListQuery
# mapDashboardInstanceSessionsCreateBody
# mapDashboardInstanceSessionsCreateOutput
# mapDashboardInstanceSessionsDeleteOutput
# mapDashboardInstanceSessionsGetOutput
# mapDashboardInstanceSessionsListOutput
# mapDashboardInstanceSessionsListQuery
# mapDashboardInstanceSessionsMessagesGetOutput
# mapDashboardInstanceSessionsMessagesListOutput
# mapDashboardInstanceSessionsMessagesListQuery
# mapDashboardOrganizationsTeamsCreateBody
# mapDashboardOrganizationsTeamsCreateOutput
# mapDashboardOrganizationsTeamsGetOutput
# mapDashboardOrganizationsTeamsListOutput
# mapDashboardOrganizationsTeamsListQuery
# mapDashboardOrganizationsTeamsMembersCreateBody
# mapDashboardOrganizationsTeamsMembersCreateOutput
# mapDashboardOrganizationsTeamsMembersDeleteOutput
# mapDashboardOrganizationsTeamsPermissionsOutput
# mapDashboardOrganizationsTeamsProjectsRemoveOutput
# mapDashboardOrganizationsTeamsProjectsSetBody
# mapDashboardOrganizationsTeamsProjectsSetOutput
# mapDashboardOrganizationsTeamsRolesCreateBody
# mapDashboardOrganizationsTeamsRolesCreateOutput
# mapDashboardOrganizationsTeamsRolesGetOutput
# mapDashboardOrganizationsTeamsRolesListOutput
# mapDashboardOrganizationsTeamsRolesListQuery
# mapDashboardOrganizationsTeamsRolesUpdateBody
# mapDashboardOrganizationsTeamsRolesUpdateOutput
# mapDashboardOrganizationsTeamsUpdateBody
# mapDashboardOrganizationsTeamsUpdateOutput

# All types and mappers are available via:
# from .resources import *
# from .endpoints import *



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class SessionsConnectionsListOutputItemsMcpClient:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class SessionsConnectionsListOutputItemsMcpServer:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class SessionsConnectionsListOutputItemsMcp:
  object: str
  version: str
  connection_type: str
  client: Optional[SessionsConnectionsListOutputItemsMcpClient] = None
  server: Optional[SessionsConnectionsListOutputItemsMcpServer] = None


@dataclass
class SessionsConnectionsListOutputItemsUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class SessionsConnectionsListOutputItemsServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class SessionsConnectionsListOutputItemsSessionUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class SessionsConnectionsListOutputItemsSession:
  object: str
  id: str
  status: str
  connection_status: str
  usage: SessionsConnectionsListOutputItemsSessionUsage
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime


@dataclass
class SessionsConnectionsListOutputItemsServerDeploymentServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class SessionsConnectionsListOutputItemsServerDeployment:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: SessionsConnectionsListOutputItemsServerDeploymentServer
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class SessionsConnectionsListOutputItems:
  object: str
  id: str
  status: str
  mcp: SessionsConnectionsListOutputItemsMcp
  usage: SessionsConnectionsListOutputItemsUsage
  server: SessionsConnectionsListOutputItemsServer
  session: SessionsConnectionsListOutputItemsSession
  server_deployment: SessionsConnectionsListOutputItemsServerDeployment
  created_at: datetime
  started_at: datetime
  ended_at: Optional[datetime] = None


@dataclass
class SessionsConnectionsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class SessionsConnectionsListOutput:
  items: List[SessionsConnectionsListOutputItems]
  pagination: SessionsConnectionsListOutputPagination


class mapSessionsConnectionsListOutputItemsMcpClient:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsMcpClient:
    return SessionsConnectionsListOutputItemsMcpClient(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsMcpClient, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsMcpServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsMcpServer:
    return SessionsConnectionsListOutputItemsMcpServer(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsMcpServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsMcp:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsMcp:
    return SessionsConnectionsListOutputItemsMcp(
      object=data.get("object"),
      version=data.get("version"),
      connection_type=data.get("connection_type"),
      client=mapSessionsConnectionsListOutputItemsMcpClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      server=mapSessionsConnectionsListOutputItemsMcpServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsMcp, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsUsage:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsUsage:
    return SessionsConnectionsListOutputItemsUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsUsage, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsServer:
    return SessionsConnectionsListOutputItemsServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsSessionUsage:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsSessionUsage:
    return SessionsConnectionsListOutputItemsSessionUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsSessionUsage, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsSession:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItemsSession:
    return SessionsConnectionsListOutputItemsSession(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      connection_status=data.get("connection_status"),
      usage=mapSessionsConnectionsListOutputItemsSessionUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItemsSession, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsServerDeploymentServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> SessionsConnectionsListOutputItemsServerDeploymentServer:
    return SessionsConnectionsListOutputItemsServerDeploymentServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      SessionsConnectionsListOutputItemsServerDeploymentServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItemsServerDeployment:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> SessionsConnectionsListOutputItemsServerDeployment:
    return SessionsConnectionsListOutputItemsServerDeployment(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapSessionsConnectionsListOutputItemsServerDeploymentServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      SessionsConnectionsListOutputItemsServerDeployment, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputItems:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputItems:
    return SessionsConnectionsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      mcp=mapSessionsConnectionsListOutputItemsMcp.from_dict(data.get("mcp"))
      if data.get("mcp")
      else None,
      usage=mapSessionsConnectionsListOutputItemsUsage.from_dict(data.get("usage"))
      if data.get("usage")
      else None,
      server=mapSessionsConnectionsListOutputItemsServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
      session=mapSessionsConnectionsListOutputItemsSession.from_dict(
        data.get("session")
      )
      if data.get("session")
      else None,
      server_deployment=mapSessionsConnectionsListOutputItemsServerDeployment.from_dict(
        data.get("server_deployment")
      )
      if data.get("server_deployment")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      started_at=parse_iso_datetime(data.get("started_at"))
      if data.get("started_at")
      else None,
      ended_at=parse_iso_datetime(data.get("ended_at"))
      if data.get("ended_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputItems, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutputPagination:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutputPagination:
    return SessionsConnectionsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutputPagination, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapSessionsConnectionsListOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListOutput:
    return SessionsConnectionsListOutput(
      items=[
        mapSessionsConnectionsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapSessionsConnectionsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class SessionsConnectionsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None


class mapSessionsConnectionsListQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> SessionsConnectionsListQuery:
    return SessionsConnectionsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
    )

  @staticmethod
  def to_dict(
    value: Union[SessionsConnectionsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ProviderOauthTakeoutsGetOutput:
  object: str
  id: str
  status: str
  metadata: Dict[str, Any]
  created_at: datetime
  note: Optional[str] = None
  access_token: Optional[str] = None
  id_token: Optional[str] = None
  scope: Optional[str] = None
  expires_at: Optional[datetime] = None


class mapProviderOauthTakeoutsGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ProviderOauthTakeoutsGetOutput:
    return ProviderOauthTakeoutsGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      note=data.get("note"),
      metadata=data.get("metadata"),
      access_token=data.get("access_token"),
      id_token=data.get("id_token"),
      scope=data.get("scope"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      expires_at=parse_iso_datetime(data.get("expires_at"))
      if data.get("expires_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ProviderOauthTakeoutsGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapDashboardOrganizationsTeamsPermissionsOutput,
  DashboardOrganizationsTeamsPermissionsOutput,
  mapDashboardOrganizationsTeamsListOutput,
  DashboardOrganizationsTeamsListOutput,
  mapDashboardOrganizationsTeamsListQuery,
  DashboardOrganizationsTeamsListQuery,
  mapDashboardOrganizationsTeamsGetOutput,
  DashboardOrganizationsTeamsGetOutput,
  mapDashboardOrganizationsTeamsUpdateOutput,
  DashboardOrganizationsTeamsUpdateOutput,
  mapDashboardOrganizationsTeamsUpdateBody,
  DashboardOrganizationsTeamsUpdateBody,
  mapDashboardOrganizationsTeamsCreateOutput,
  DashboardOrganizationsTeamsCreateOutput,
  mapDashboardOrganizationsTeamsCreateBody,
  DashboardOrganizationsTeamsCreateBody,
)


class MetorialManagementOrganizationTeamsEndpoint(BaseMetorialEndpoint):
  """Read and write team information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def permissions(self) -> DashboardOrganizationsTeamsPermissionsOutput:
    """
    Get team
    Get the information of a specific team


    :return: DashboardOrganizationsTeamsPermissionsOutput
    """
    request = MetorialRequest(path=["organization", "team-role-permissions"])
    return self._get(request).transform(
      mapDashboardOrganizationsTeamsPermissionsOutput.from_dict
    )

  def list(
    self,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None
  ) -> DashboardOrganizationsTeamsListOutput:
    """
    List organization teams
    List all organization teams

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: DashboardOrganizationsTeamsListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order

    request = MetorialRequest(path=["organization", "teams"], query=query_dict)
    return self._get(request).transform(
      mapDashboardOrganizationsTeamsListOutput.from_dict
    )

  def get(self, team_id: str) -> DashboardOrganizationsTeamsGetOutput:
    """
    Get team
    Get the information of a specific team

    :param team_id: str
    :return: DashboardOrganizationsTeamsGetOutput
    """
    request = MetorialRequest(path=["organization", "teams", team_id])
    return self._get(request).transform(
      mapDashboardOrganizationsTeamsGetOutput.from_dict
    )

  def update(
    self, team_id: str, *, name: Optional[str] = None, description: Optional[str] = None
  ) -> DashboardOrganizationsTeamsUpdateOutput:
    """
    Update team
    Update the role of an team

    :param team_id: str
    :param name: Optional[str] (optional)
    :param description: Optional[str] (optional)
    :return: DashboardOrganizationsTeamsUpdateOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    if name is not None:
      body_dict["name"] = name
    if description is not None:
      body_dict["description"] = description

    request = MetorialRequest(path=["organization", "teams", team_id], body=body_dict)
    return self._post(request).transform(
      mapDashboardOrganizationsTeamsUpdateOutput.from_dict
    )

  def create(
    self, *, name: str, description: Optional[str] = None
  ) -> DashboardOrganizationsTeamsCreateOutput:
    """
    Create organization team
    Create a new organization team

    :param name: str
    :param description: Optional[str] (optional)
    :return: DashboardOrganizationsTeamsCreateOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    body_dict["name"] = name
    if description is not None:
      body_dict["description"] = description

    request = MetorialRequest(path=["organization", "teams"], body=body_dict)
    return self._post(request).transform(
      mapDashboardOrganizationsTeamsCreateOutput.from_dict
    )



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputDeployment:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputConfig:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputAuthConfig:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger:
    object: str
    id: str
    key: str
    name: str
    invocation: Dict[str, Any]
    provider_id: str
    provider_specification_id: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    input_schema: Optional[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema] = None
    output_schema: Optional[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema] = None
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutputTriggers:
    object: str
    id: str
    source: str
    poll_interval_seconds: Optional[float] = None
    next_poll_at: Optional[datetime] = None
    last_polled_at: Optional[datetime] = None
    webhook_url: Optional[str] = None
    is_webhook_registered: Optional[bool] = None
    provider_trigger: Optional[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger] = None
@dataclass
class ManagementInstanceCallbacksInstancesCreateOutput:
    object: str
    id: str
    status: str
    deployment: ManagementInstanceCallbacksInstancesCreateOutputDeployment
    config: ManagementInstanceCallbacksInstancesCreateOutputConfig
    triggers: List[ManagementInstanceCallbacksInstancesCreateOutputTriggers]
    created_at: datetime
    updated_at: datetime
    auth_config: Optional[ManagementInstanceCallbacksInstancesCreateOutputAuthConfig] = None


class mapManagementInstanceCallbacksInstancesCreateOutputDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputDeployment:
        return ManagementInstanceCallbacksInstancesCreateOutputDeployment(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputConfig:
        return ManagementInstanceCallbacksInstancesCreateOutputConfig(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputAuthConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputAuthConfig:
        return ManagementInstanceCallbacksInstancesCreateOutputAuthConfig(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputAuthConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema:
        return ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema:
        return ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger:
        return ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger(
        object=data.get('object'),
        id=data.get('id'),
        key=data.get('key'),
        name=data.get('name'),
        description=data.get('description'),
        input_schema=mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerInputSchema.from_dict(data.get('input_schema')) if data.get('input_schema') else None,
        output_schema=mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTriggerOutputSchema.from_dict(data.get('output_schema')) if data.get('output_schema') else None,
        invocation=data.get('invocation'),
        provider_id=data.get('provider_id'),
        provider_specification_id=data.get('provider_specification_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutputTriggers:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutputTriggers:
        return ManagementInstanceCallbacksInstancesCreateOutputTriggers(
        object=data.get('object'),
        id=data.get('id'),
        source=data.get('source'),
        poll_interval_seconds=data.get('poll_interval_seconds'),
        next_poll_at=datetime.fromisoformat(data.get('next_poll_at').replace('Z', '+00:00')) if data.get('next_poll_at') else None,
        last_polled_at=datetime.fromisoformat(data.get('last_polled_at').replace('Z', '+00:00')) if data.get('last_polled_at') else None,
        webhook_url=data.get('webhook_url'),
        is_webhook_registered=data.get('is_webhook_registered'),
        provider_trigger=mapManagementInstanceCallbacksInstancesCreateOutputTriggersProviderTrigger.from_dict(data.get('provider_trigger')) if data.get('provider_trigger') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutputTriggers, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCallbacksInstancesCreateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateOutput:
        return ManagementInstanceCallbacksInstancesCreateOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        deployment=mapManagementInstanceCallbacksInstancesCreateOutputDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        config=mapManagementInstanceCallbacksInstancesCreateOutputConfig.from_dict(data.get('config')) if data.get('config') else None,
        auth_config=mapManagementInstanceCallbacksInstancesCreateOutputAuthConfig.from_dict(data.get('auth_config')) if data.get('auth_config') else None,
        triggers=[mapManagementInstanceCallbacksInstancesCreateOutputTriggers.from_dict(item) for item in data.get('triggers', []) if item],
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ManagementInstanceCallbacksInstancesCreateBody:
    provider_config_id: str
    provider_auth_config_id: Optional[str] = None


class mapManagementInstanceCallbacksInstancesCreateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCallbacksInstancesCreateBody:
        return ManagementInstanceCallbacksInstancesCreateBody(
        provider_config_id=data.get('provider_config_id'),
        provider_auth_config_id=data.get('provider_auth_config_id')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCallbacksInstancesCreateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

